@@ -75,6 +75,7 @@ static struct {
   { 246, 185, 1024,  768, "FUJITSU SIEMENS", "LifeBook T Series", NULL },
   { 304, 190, 1400, 1050, "Hewlett-Packard", "HP Compaq nc6000 (PN887PA#ABG)", "F.0F" },
   { 332, 208, 1280,  800, "MICRO-STAR INT'L CO.,LTD.", "MS-1011", "0101" },
+  { 304, 189, 1440,  900, "Dell Inc.", "Latitude D630C", NULL },
 };
 
 #define BIOS_TEST
@@ -92,6 +93,7 @@ static void get_fsc_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *b
 static void add_panel_info(hd_data_t *hd_data, bios_info_t *bt);
 #endif
 static void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt);
+static void chk_vbox(hd_data_t *hd_data);
 static unsigned char crc(unsigned char *mem, unsigned len);
 static int get_smp_info(hd_data_t *hd_data, memory_range_t *mem, smp_info_t *smp);
 static unsigned parse_mpconfig_len(hd_data_t *hd_data, memory_range_t *mem);
@@ -355,6 +357,7 @@ void hd_scan_bios(hd_data_t *hd_data)
     add_panel_info(hd_data, bt);
 #endif
     add_mouse_info(hd_data, bt);
+    chk_vbox(hd_data);
   }
 
   PROGRESS(3, 0, "smp");
@@ -938,6 +941,22 @@ void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt)
 }
 
 
+void chk_vbox(hd_data_t *hd_data)
+{
+  hd_smbios_t *sm;
+
+  for(sm = hd_data->smbios; sm; sm = sm->next) {
+    if(
+      sm->any.type == sm_sysinfo &&
+      sm->sysinfo.product &&
+      !strcmp(sm->sysinfo.product, "VirtualBox")
+    ) {
+      hd_data->flags.vbox = 1;
+    }
+  }
+}
+
+
 int get_smp_info(hd_data_t *hd_data, memory_range_t *mem, smp_info_t *smp)
 {
 #ifndef __ia64__
