@@ -54,7 +54,8 @@ void hd_scan_sysfs_block(hd_data_t *hd_data)
 
   if(hd_probe_feature(hd_data, pr_block_mods)) {
     PROGRESS(1, 0, "block modules");
-    load_module(hd_data, "ide-cd");
+    // load_module(hd_data, "ide-cd");
+    load_module(hd_data, "ide-cd_mod");
     load_module(hd_data, "ide-disk");
     load_module(hd_data, "sr_mod");
     load_module(hd_data, "sd_mod");
@@ -97,6 +98,7 @@ void get_block_devs(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev = NULL, *sf_dev_ide;
   char *sf_drv_name, *sf_drv, *bus_id, *bus_name, *ide_bus_id;
   str_list_t *sf_bus, *sf_bus_e;
+  char *sf_block_dir;
 
   sf_bus = reverse_str_list(read_dir("/sys/bus/ide/devices", 'l'));
 
@@ -112,7 +114,18 @@ void get_block_devs(hd_data_t *hd_data)
     }
   }
 
-  sf_class = reverse_str_list(read_dir("/sys/block", 'd'));
+  sf_block_dir = "/sys/subsystem/block";
+  sf_class = reverse_str_list(read_dir(sf_block_dir, 'l'));
+
+  if (!sf_class) {
+    sf_block_dir = "/sys/class/block";
+    sf_class = reverse_str_list(read_dir(sf_block_dir, 'l'));
+  }
+
+  if (!sf_class) {
+    sf_block_dir = "/sys/block";
+    sf_class = reverse_str_list(read_dir(sf_block_dir, 'd'));
+  }
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: block\n");
@@ -120,8 +133,7 @@ void get_block_devs(hd_data_t *hd_data)
   }
 
   for(sf_class_e = sf_class; sf_class_e; sf_class_e = sf_class_e->next) {
-    str_printf(&sf_cdev, 0, "/sys/block/%s", sf_class_e->str);
-
+    str_printf(&sf_cdev, 0, "%s/%s", sf_block_dir, sf_class_e->str);
     ADD2LOG(
       "  block: name = %s, path = %s\n",
       sf_class_e->str,
@@ -812,7 +824,7 @@ void add_scsi_sysfs_info(hd_data_t *hd_data, hd_t *hd, char *sf_dev)
     hd_report_this(hd_data, hd) &&
     hd->unix_dev_name &&
     hd->sub_class.id == sc_sdev_cdrom &&
-    hd_data->in_vmware != 1		/* VMWare doesn't like it */
+    !hd_data->flags.vmware		/* VMWare doesn't like it */
   ) {
     PROGRESS(5, 0, hd->unix_dev_name);
     fd = open(hd->unix_dev_name, O_RDONLY | O_NONBLOCK);
@@ -1331,7 +1343,7 @@ void get_scsi_tape(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev = NULL;
   char *sf_drv_name, *sf_drv, *bus_id;
 
-  sf_class = reverse_str_list(read_dir("/sys/class/scsi_tape", 'd'));
+  sf_class = reverse_str_list(read_dir("/sys/class/scsi_tape", 'D'));
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: scsi_tape\n");
@@ -1467,7 +1479,7 @@ void get_generic_scsi_devs(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev = NULL;
   char *sf_drv_name, *sf_drv, *bus_id;
 
-  sf_class = reverse_str_list(read_dir("/sys/class/scsi_generic", 'd'));
+  sf_class = reverse_str_list(read_dir("/sys/class/scsi_generic", 'D'));
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: scsi_generic\n");
