@@ -21,7 +21,12 @@
 
 #if defined(__i386__) || defined(__x86_64__)
 
-static void get_edd_info(hd_data_t *hd_data);
+static void read_edd_info(hd_data_t *hd_data);
+static int does_match(edd_info_t *ei, hd_t *hd, unsigned type);
+static int does_match0(edd_info_t *ei, edd_info_t *ei0, unsigned type);
+static int is_disk(hd_t *hd);
+static uint64_t disk_size(hd_t *hd);
+static int identical_disks(hd_t *hd1, hd_t *hd2);
 
 void hd_scan_sysfs_edd(hd_data_t *hd_data)
 {
@@ -41,21 +46,17 @@ void hd_scan_sysfs_edd(hd_data_t *hd_data)
 
   PROGRESS(2, 0, "edd info");
 
-  get_edd_info(hd_data);
+  read_edd_info(hd_data);
 }
 
 
-void get_edd_info(hd_data_t *hd_data)
+void read_edd_info(hd_data_t *hd_data)
 {
-  hd_t *hd;
-  hd_res_t *res;
-  unsigned u, u1, u2, edd_cnt = 0, lba;
+  unsigned u;
   uint64_t ul0;
-  str_list_t *sl;
-  bios_info_t *bt;
   edd_info_t *ei;
-  str_list_t *sf_dir, *sf_dir_e;
-  char *sf_edd = NULL, *sf_link;
+  str_list_t *sl, *sf_dir, *sf_dir_e;
+  char *sf_edd = NULL;
 
   for(u = 0; u < sizeof hd_data->edd / sizeof *hd_data->edd; u++) {
     free_mem(hd_data->edd[u].sysfs_id);
@@ -73,12 +74,12 @@ void get_edd_info(hd_data_t *hd_data)
       u >= 0x80 &&
       u <= 0xff
     ) {
-      edd_cnt++;
-
       u -= 0x80;
 
       ei = hd_data->edd + u;
 
+      ei->valid = 1;
+
       if(hd_attr_uint(get_sysfs_attr_by_path(sf_edd, "sectors"), &ul0, 0)) {
         ei->sectors = ul0;
       }
@@ -111,12 +112,8 @@ void get_edd_info(hd_data_t *hd_data)
         ei->edd.cyls = ei->sectors / (ei->edd.heads * ei->edd.sectors);
       }
 
-      sf_link = hd_read_sysfs_link(sf_edd, "pci_dev");
-      if(sf_link) {
-        hd_data->edd[u].sysfs_id = new_str(hd_sysfs_id(sf_link));
-        if((hd = hd_find_sysfs_id(hd_data, hd_data->edd[u].sysfs_id))) {
-          hd_data->edd[u].hd_idx = hd->idx;
-        }
+      if(hd_attr_uint(get_sysfs_attr_by_path(sf_edd, "mbr_signature"), &ul0, 0)) {
+        ei->signature = ul0;
       }
 
       sl = hd_attr_list(get_sysfs_attr_by_path(sf_edd, "extensions"));
@@ -126,8 +123,9 @@ void get_edd_info(hd_data_t *hd_data)
       if(search_str_list(sl, "64-bit extensions")) hd_data->edd[u].ext_64bit = 1;
 
       ADD2LOG(
-        "edd: 0x%02x\n  size: %"PRIu64"\n  chs default: %u/%u/%u\n  chs legacy: %u/%u/%u\n  caps: %s%s%s%s\n  attached: #%u %s\n",
+        "edd: 0x%02x\n  mbr sig: 0x%08x\n  size: %"PRIu64"\n  chs default: %u/%u/%u\n  chs legacy: %u/%u/%u\n  caps: %s%s%s%s\n",
         u + 0x80,
+        ei->signature,
         ei->sectors,
         ei->edd.cyls,
         ei->edd.heads,
@@ -138,78 +136,79 @@ void get_edd_info(hd_data_t *hd_data)
         ei->ext_fixed_disk ? "fixed " : "",
         ei->ext_lock_eject ? "lock " : "",
         ei->ext_edd ? "edd " : "",
-        ei->ext_64bit ? "64bit " : "",
-        ei->hd_idx,
-        ei->sysfs_id ?: ""
+        ei->ext_64bit ? "64bit " : ""
       );
     }
   }
 
   free_mem(sf_edd);
   free_str_list(sf_dir);
+}
 
-  if(!edd_cnt) return;
+
+void assign_edd_info(hd_data_t *hd_data)
+{
+  hd_t *hd, *match_hd;
+  unsigned u, u1, u2, lba, matches, real_matches, match_edd, type;
+  bios_info_t *bt;
+  edd_info_t *ei;
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
-    if(
-      hd->base_class.id == bc_storage_device &&
-      hd->sub_class.id == sc_sdev_disk
-    ) {
-      hd->rom_id = free_mem(hd->rom_id);
-    }
+    if(is_disk(hd)) hd->rom_id = free_mem(hd->rom_id);
   }
 
   /* add BIOS drive ids to disks */
+  for(type = 0; type < 4; type++) {
+    for(u = 0; u < sizeof hd_data->edd / sizeof *hd_data->edd; u++) {
+      ei = hd_data->edd + u;
+      if(!ei->valid || ei->assigned) continue;
 
-  /* first, check sysfs link */
-  for(u = 0; u < sizeof hd_data->edd / sizeof *hd_data->edd; u++) {
-    if(!hd_data->edd[u].hd_idx) continue;
-    for(hd = hd_data->hd; hd; hd = hd->next) {
-      if(
-        hd->base_class.id == bc_storage_device &&
-        hd->sub_class.id == sc_sdev_disk &&
-        hd->attached_to == hd_data->edd[u].hd_idx &&
-        !hd->rom_id
-      ) {
-        str_printf(&hd->rom_id, 0, "0x%02x", u + 0x80);
-        hd_data->flags.edd_used = 1;
-        hd_data->edd[u].assigned = 1;
-        break;
+      for(u1 = u2 = 0; u1 < sizeof hd_data->edd / sizeof *hd_data->edd; u1++) {
+        if(does_match0(ei, hd_data->edd + u1, type)) u2++;
       }
-    }
-  }
 
-  /* try based on disk size */
-  for(u = 0; u < sizeof hd_data->edd / sizeof *hd_data->edd; u++) {
-    if(hd_data->edd[u].assigned) continue;
-    if(!(ul0 = hd_data->edd[u].sectors)) continue;
-    for(u1 = u2 = 0; u1 < sizeof hd_data->edd / sizeof *hd_data->edd; u1++) {
-      if(ul0 == hd_data->edd[u1].sectors) u2++;
-    }
+      /* not unique */
+      if(u2 != 1) continue;
 
-    /* more than one disk with this size */
-    if(u2 != 1) continue;
+      matches = real_matches = 0;
+      match_hd = 0;
+      match_edd = 0;
 
-    for(hd = hd_data->hd; hd; hd = hd->next) {
-      if(
-        hd->base_class.id == bc_storage_device &&
-        hd->sub_class.id == sc_sdev_disk &&
-        !hd->rom_id
-      ) {
-        for(res = hd->res; res; res = res->next) {
-          if(
-            res->any.type == res_size &&
-            res->size.unit == size_unit_sectors &&
-            res->size.val1 == ul0
-          ) break;
+      for(hd = hd_data->hd; hd; hd = hd->next) {
+        if(!is_disk(hd) || hd->rom_id) continue;
+
+        if(does_match(ei, hd, type)) {
+          if(!matches) {
+            match_edd = u;
+            match_hd = hd;
+          }
+          else {
+            if(identical_disks(hd, match_hd)) real_matches--;
+          }
+          matches++;
+          real_matches++;
         }
+      }
 
-        if(!res) continue;
+      ADD2LOG("  %02x: matches %d (%d)\n", u + 0x80, real_matches, matches);
 
-        str_printf(&hd->rom_id, 0, "0x%02x", u + 0x80);
+      if(real_matches == 1) {
         hd_data->flags.edd_used = 1;
-        hd_data->edd[u].assigned = 1;
-        break;
+        hd_data->edd[match_edd].assigned = 1;
+
+        if(matches == 1) {
+          str_printf(&match_hd->rom_id, 0, "0x%02x", match_edd + 0x80);
+          ADD2LOG("  %s = %s (match %d)\n", match_hd->unix_dev_name, match_hd->rom_id, type);
+        }
+        else {
+          for(hd = hd_data->hd; hd; hd = hd->next) {
+            if(!is_disk(hd) || hd->rom_id) continue;
+            if(does_match(ei, hd, type)) {
+              str_printf(&hd->rom_id, 0, "0x%02x", match_edd + 0x80);
+              ADD2LOG("  %s = %s (match %d)\n", hd->unix_dev_name, hd->rom_id, type);
+            }
+          }
+        }
       }
     }
   }
@@ -235,9 +234,130 @@ void get_edd_info(hd_data_t *hd_data)
       }
     }
   }
-
 }
 
+
+int does_match(edd_info_t *ei, hd_t *hd, unsigned type)
+{
+  int i = 0;
+  uint64_t u64;
+
+  switch(type) {
+    case 0:
+      i = ei->signature == edd_disk_signature(hd) && ei->sectors == disk_size(hd);
+      break;
+
+    case 1:
+      i = ei->signature == edd_disk_signature(hd);
+      break;
+
+    case 2:
+      i = ei->sectors == disk_size(hd);
+      break;
+
+    case 3:
+      u64 = ei->edd.heads * ei->edd.sectors;
+      if(u64) {
+        i = ei->edd.cyls == disk_size(hd) / u64;
+      }
+      break;
+  }
+
+  return i;
+}
+
+
+int does_match0(edd_info_t *ei, edd_info_t *ei0, unsigned type)
+{
+  int i = 0;
+
+  switch(type) {
+    case 0:
+      i = ei->signature == ei0->signature && ei->sectors == ei0->sectors;
+      break;
+
+    case 1:
+      i = ei->signature == ei0->signature;
+      break;
+
+    case 2:
+      i = ei->sectors == ei0->sectors;
+      break;
+
+    case 3:
+      i = ei->edd.cyls == ei0->edd.cyls;
+      break;
+  }
+
+  return i;
+}
+
+
+int is_disk(hd_t *hd)
+{
+  return
+    hd->base_class.id == bc_storage_device &&
+    hd->sub_class.id == sc_sdev_disk;
+}
+
+
+uint64_t disk_size(hd_t *hd)
+{
+  hd_res_t *res;
+  uint64_t ul = 0;
+
+  for(res = hd->res; res; res = res->next) {
+    if(
+      res->any.type == res_size &&
+      res->size.unit == size_unit_sectors
+    ) {
+      ul = res->size.val1;
+      break;
+    }
+  }
+
+  return ul;
+}
+
+
+unsigned edd_disk_signature(hd_t *hd)
+{
+  unsigned char *s = hd->block0;
+
+  if(!s) return 0;
+
+  return s[0x1b8] + (s[0x1b9] << 8) + (s[0x1ba] << 16) + (s[0x1bb] << 24);
+}
+
+
+int identical_disks(hd_t *hd1, hd_t *hd2)
+{
+  char *s1 = NULL, *s2 = NULL;
+  str_list_t *sl;
+
+  for(sl = hd1->unix_dev_names; sl; sl = sl->next) {
+    if(!strncmp(sl->str, "/dev/disk/by-id/", sizeof "/dev/disk/by-id/" - 1)) {
+      s1 = sl->str;
+      break;
+    }
+  }
+
+  for(sl = hd2->unix_dev_names; sl; sl = sl->next) {
+    if(!strncmp(sl->str, "/dev/disk/by-id/", sizeof "/dev/disk/by-id/" - 1)) {
+      s2 = sl->str;
+      break;
+    }
+  }
+
+  if(!s1) s1 = hd1->serial;
+  if(!s2) s2 = hd1->serial;
+
+  if(s1 && s2 && !strcmp(s1, s2)) return 1;
+
+  return 0;
+}
+
+
 #endif /* defined(__i386__) || defined(__x86_64__) */
 
 /** @} */
