@@ -23,8 +23,6 @@
 #define PROC_MEMINFO		"/proc/meminfo"
 #define PROC_VERSION		"/proc/version"
 #define PROC_ISAPNP		"/proc/isapnp"
-#define PROC_ISERIES		"/proc/iSeries"
-#define PROC_ISERIES_VETH	"/proc/iSeries/veth"
 #define PROC_PARTITIONS		"/proc/partitions"
 #define PROC_APM		"/proc/apm"
 
@@ -211,6 +209,7 @@ void hd_move_to_shm(hd_data_t *hd_data);
 void read_udevinfo(hd_data_t *hd_data);
 
 hd_t *hd_find_sysfs_id(hd_data_t *hd_data, char *id);
+hd_t *hd_find_sysfs_id_devname(hd_data_t *hd_data, char *id, char *devname);
 int hd_attr_uint(char* attr, uint64_t* u, int base);
 str_list_t *hd_attr_list(char *str);
 char *hd_sysfs_id(char *path);
@@ -224,7 +223,6 @@ str_list_t *hd_module_list(hd_data_t *hd_data, unsigned id);
 char* get_sysfs_attr(const char* bus, const char* device, const char* attr);
 char* get_sysfs_attr_by_path(const char* path, const char* attr);
 
-int hd_is_iseries(hd_data_t *hd_data);
 hal_device_t *hd_free_hal_devices(hal_device_t *dev);
 void hd_pci_complete_data(hd_t *hd);
 void hd_pci_read_data(hd_data_t *hd_data);
