@@ -9,6 +9,7 @@
 #include "hd.h"
 #include "hd_int.h"
 #include "int.h"
+#include "edd.h"
 
 /**
  * @defgroup LIBHDint Internal utilities
@@ -69,7 +70,10 @@ void hd_scan_int(hd_data_t *hd_data)
   int_floppy(hd_data);
 
 #if defined(__i386__) || defined (__x86_64__)
-  PROGRESS(5, 0, "bios");
+  PROGRESS(5, 0, "edd");
+  assign_edd_info(hd_data);
+
+  PROGRESS(5, 1, "bios");
   int_bios(hd_data);
 #endif
 
@@ -489,6 +493,11 @@ void int_media_check(hd_data_t *hd_data)
       PROGRESS(4, ++j, hd->unix_dev_name);
       hd->block0 = read_block0(hd_data, hd->unix_dev_name, &i);
       hd->is.notready = hd->block0 ? 0 : 1;
+#if defined(__i386__) || defined(__x86_64__)
+      if(hd->block0) {
+        ADD2LOG("  mbr sig: 0x%08x\n", edd_disk_signature(hd));
+      }
+#endif
     }
   }
 }
@@ -665,6 +674,8 @@ void int_mouse(hd_data_t *hd_data)
 {
   hd_t *hd;
   bios_info_t *bt = NULL;
+
+#if 0
   str_list_t *sl;
   int is_mouse;
 
@@ -702,6 +713,32 @@ void int_mouse(hd_data_t *hd_data)
       hd->compat_device.id = 0;
     }
   }
+#endif
+
+  if(hd_data->flags.vbox || hd_data->flags.vmware) {
+    for(hd = hd_data->hd; hd; hd = hd->next) {
+      if(
+        hd->base_class.id == bc_mouse &&
+        hd->sub_class.id == sc_mou_ps2 &&
+        hd->vendor.id == 2
+      ) {
+        if(hd_data->flags.vbox && hd->device.id == 6) {
+          hd->vendor.name = free_mem(hd->vendor.name);
+          hd->device.name = free_mem(hd->device.name);
+          hd->vendor.id = MAKE_ID(TAG_SPECIAL, 0x80ee);
+          hd->device.id = MAKE_ID(TAG_SPECIAL, 0x0100);
+          new_id(hd_data, hd);
+        }
+        else if(hd_data->flags.vmware && hd->device.id == 5) {
+          hd->vendor.name = free_mem(hd->vendor.name);
+          hd->device.name = free_mem(hd->device.name);
+          hd->vendor.id = MAKE_ID(TAG_SPECIAL, 0x15ad);
+          hd->device.id = MAKE_ID(TAG_SPECIAL, 0x0100);
+          new_id(hd_data, hd);
+        }
+      }
+    }
+  }
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
     if(
@@ -920,7 +957,7 @@ void int_udev(hd_data_t *hd_data)
           }
         }
 
-        if(!hd->unix_dev_name) {
+        if(!hd->unix_dev_name || hd_data->flags.udev) {
           sl = hd->unix_dev_names;
 
           if(hd_data->flags.udev) {
