@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <inttypes.h>
@@ -36,8 +37,6 @@
 static void get_driverinfo(hd_data_t *hd_data, hd_t *hd);
 static void get_linkstate(hd_data_t *hd_data, hd_t *hd);
 static void add_xpnet(hd_data_t *hdata);
-static void add_mv(hd_data_t *hdata);
-static void add_iseries(hd_data_t *hdata);
 static void add_uml(hd_data_t *hdata);
 static void add_kma(hd_data_t *hdata);
 static void add_if_name(hd_t *hd_card, hd_t *hd);
@@ -51,7 +50,7 @@ void hd_scan_net(hd_data_t *hd_data)
   unsigned u;
   int if_type, if_carrier;
   hd_t *hd, *hd_card;
-  char *s, *hw_addr;
+  char *s, *t, *hw_addr;
   hd_res_t *res, *res1, *res2;
   uint64_t ul0;
   str_list_t *sf_class, *sf_class_e;
@@ -152,77 +151,6 @@ void hd_scan_net(hd_data_t *hd_data)
       get_driverinfo(hd_data, hd);
     }
 
-    hd_card = NULL;
-
-    if(sf_dev) {
-      hd->sysfs_device_link = new_str(hd_sysfs_id(sf_dev)); 
-
-      hd_card = hd_find_sysfs_id(hd_data, hd_sysfs_id(sf_dev));
-      if(hd_card) {
-        hd->attached_to = hd_card->idx;
-
-        /* for cards with strange pci classes */
-        hd_set_hw_class(hd_card, hw_network_ctrl);
-
-        /* add hw addr to network card */
-        if(res1) {
-          u = 0;
-          for(res = hd_card->res; res; res = res->next) {
-            if(
-              res->any.type == res_hwaddr &&
-              !strcmp(res->hwaddr.addr, res1->hwaddr.addr)
-            ) {
-              u = 1;
-              break;
-            }
-          }
-          if(!u) {
-            res = new_mem(sizeof *res);
-            res->hwaddr.type = res_hwaddr;
-            res->hwaddr.addr = new_str(res1->hwaddr.addr);
-            add_res_entry(&hd_card->res, res);
-          }
-        }
-        /* add interface names */
-        add_if_name(hd_card, hd);
-      }
-    }
-
-    if(!hd_card && hw_addr) {
-      /* try to find card based on hwaddr (for prom-based cards) */
-
-      for(hd_card = hd_data->hd; hd_card; hd_card = hd_card->next) {
-        if(
-          hd_card->base_class.id != bc_network ||
-          hd_card->sub_class.id != 0
-        ) continue;
-        for(res = hd_card->res; res; res = res->next) {
-          if(
-            res->any.type == res_hwaddr &&
-            !strcmp(hw_addr, res->hwaddr.addr)
-          ) break;
-        }
-        if(res) {
-          hd->attached_to = hd_card->idx;
-          break;
-        }
-      }
-    }
-
-    hw_addr = free_mem(hw_addr);
-
-#if 0
-    "ctc"	sc_nif_ctc
-    "iucv"	sc_nif_iucv
-    "hsi"	sc_nif_hsi
-    "qeth"	sc_nif_qeth
-    "escon"	sc_nif_escon
-    "myri"	sc_nif_myrinet
-    "wlan"	sc_nif_wlan
-    "xp"	sc_nif_xp
-    "usb"	sc_nif_usb
-#endif
-
     switch(if_type) {
       case ARPHRD_ETHER:	/* eth */
         hd->sub_class.id = sc_nif_ethernet;
@@ -244,6 +172,8 @@ void hd_scan_net(hd_data_t *hd_data)
         hd->sub_class.id = sc_nif_fc;
         break;
 #endif
+      default:
+        hd->sub_class.id = sc_nif_other;
     }
 
     if(!strcmp(hd->unix_dev_name, "lo")) {
@@ -302,9 +232,98 @@ void hd_scan_net(hd_data_t *hd_data)
       hd->slot = u;
     }
     /* ##### add more interface names here */
+    else {
+      for(s = hd->unix_dev_name; *s; s++) if(isdigit(*s)) break;
+      if(*s && (u = strtoul(s, &s, 10), !*s)) {
+        hd->slot = u;
+      }
+    }
 
     hd->bus.id = bus_none;
 
+    hd_card = NULL;
+
+    if(sf_dev) {
+      s = new_str(hd_sysfs_id(sf_dev));
+
+      hd->sysfs_device_link = new_str(s);
+
+      hd_card = hd_find_sysfs_id(hd_data, s);
+
+      // try one above, if not found
+      if(!hd_card) {
+        t = strrchr(s, '/');
+        if(t) {
+          *t = 0;
+          hd_card = hd_find_sysfs_id(hd_data, s);
+        }
+      }
+
+      /* if one card has several interfaces (as with PS3), check interface names, too */
+      if(
+        hd_card &&
+        hd_card->unix_dev_name &&
+        hd->unix_dev_name &&
+        strcmp(hd->unix_dev_name, hd_card->unix_dev_name)
+      ) {
+        hd_card = hd_find_sysfs_id_devname(hd_data, s, hd->unix_dev_name);
+      }
+
+      s = free_mem(s);
+
+      if(hd_card) {
+        hd->attached_to = hd_card->idx;
+
+        /* for cards with strange pci classes */
+        hd_set_hw_class(hd_card, hw_network_ctrl);
+
+        /* add hw addr to network card */
+        if(res1) {
+          u = 0;
+          for(res = hd_card->res; res; res = res->next) {
+            if(
+              res->any.type == res_hwaddr &&
+              !strcmp(res->hwaddr.addr, res1->hwaddr.addr)
+            ) {
+              u = 1;
+              break;
+            }
+          }
+          if(!u) {
+            res = new_mem(sizeof *res);
+            res->hwaddr.type = res_hwaddr;
+            res->hwaddr.addr = new_str(res1->hwaddr.addr);
+            add_res_entry(&hd_card->res, res);
+          }
+        }
+        /* add interface names */
+        add_if_name(hd_card, hd);
+      }
+    }
+
+    if(!hd_card && hw_addr) {
+      /* try to find card based on hwaddr (for prom-based cards) */
+
+      for(hd_card = hd_data->hd; hd_card; hd_card = hd_card->next) {
+        if(
+          hd_card->base_class.id != bc_network ||
+          hd_card->sub_class.id != 0
+        ) continue;
+        for(res = hd_card->res; res; res = res->next) {
+          if(
+            res->any.type == res_hwaddr &&
+            !strcmp(hw_addr, res->hwaddr.addr)
+          ) break;
+        }
+        if(res) {
+          hd->attached_to = hd_card->idx;
+          break;
+        }
+      }
+    }
+
+    hw_addr = free_mem(hw_addr);
+
     /* fix card type */
     if(hd_card) {
       if(
@@ -332,10 +351,8 @@ void hd_scan_net(hd_data_t *hd_data)
   sf_class = free_str_list(sf_class);
 
   if(hd_is_sgi_altix(hd_data)) add_xpnet(hd_data);
-  if(hd_is_iseries(hd_data)) add_iseries(hd_data);
   add_uml(hd_data);
   add_kma(hd_data);
-  add_mv(hd_data);
 
   /* add link status info */
   for(hd = hd_data->hd ; hd; hd = hd->next) {
@@ -485,161 +502,8 @@ void add_xpnet(hd_data_t *hd_data)
 }
 
 
-/*
- * Marvell Gigabit Ethernet thing
- */
-void add_mv(hd_data_t *hd_data)
-{
-  hd_t *hd, *hd_card;
-  hd_res_t *res, *res2;
-  struct stat sbuf;
-  char *s, *sf_dev, *sf_dev_name, *module;
-
-  /*
-   * Actually there are two (.0 & .1), but only one seems to be used - so we
-   * don't care.
-   */
-  sf_dev = "/sys/devices/platform/mv643xx_eth.0";
-  sf_dev_name = "mv643xx_eth.0";
-  module = "mv643xx_eth";
-
-  if(stat(sf_dev, &sbuf) || !S_ISDIR(sbuf.st_mode)) return;
-
-  for(hd = hd_data->hd ; hd; hd = hd->next) {
-    if(
-      hd->vendor.id == MAKE_ID(TAG_PCI, 0x11ab) &&
-      hd->device.id == MAKE_ID(TAG_PCI, 0x6460)
-    ) break;
-  }
-
-  hd_card = add_hd_entry(hd_data, __LINE__, 0);
-  hd_card->base_class.id = bc_network;
-  hd_card->sub_class.id = 0;
-
-  hd_card->sysfs_id = new_str(hd_sysfs_id(sf_dev));
-  hd_card->sysfs_bus_id = new_str(sf_dev_name);
-
-  if((s = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
-    hd_card->modalias = canon_str(s, strlen(s));
-    ADD2LOG("    modalias = \"%s\"\n", hd_card->modalias);
-  }
-  else {
-    hd_card->modalias = new_str(module);
-  }
-
-  if(hd) {
-    hd_card->attached_to = hd->idx;
-    hd_card->vendor.id = hd->vendor.id;
-    hd_card->device.id = hd->device.id;
-  }
-  else {
-    hd_card->vendor.name = new_str("Marvell");
-    hd_card->device.name = new_str("Gigabit Ethernet");
-  }
-
-  if(hd_module_is_active(hd_data, module)) {
-    add_str_list(&hd_card->drivers, module);
-  }
-
-  for(hd = hd_data->hd ; hd; hd = hd->next) {
-    if(
-      hd->module == hd_data->module &&
-      hd->base_class.id == bc_network_interface &&
-      hd->sub_class.id == sc_nif_ethernet &&
-      search_str_list(hd->drivers, "mv643xx_eth")
-    ) {
-      hd->attached_to = hd_card->idx;
-
-      for(res = hd->res; res; res = res->next) {
-        if(res->any.type == res_hwaddr) break;
-      }
-
-      if(res) {
-        res2 = new_mem(sizeof *res2);
-        res2->hwaddr.type = res_hwaddr;
-        res2->hwaddr.addr = new_str(res->hwaddr.addr);
-        add_res_entry(&hd_card->res, res2);
-      }
-
-      add_if_name(hd_card, hd);
-
-      break;
-    }
-  }
-}
 
 
-/*
- * iSeries veth devices.
- */
-void add_iseries(hd_data_t *hd_data)
-{
-  hd_t *hd, *hd_card;
-  hd_res_t *res, *res2;
-  unsigned i, cardmask = 0, card_cnt = 0;
-  str_list_t *sl0, *sl;
-
-  for(hd = hd_data->hd ; hd; hd = hd->next) {
-    if(
-      hd->module == hd_data->module &&
-      hd->base_class.id == bc_network_interface &&
-      (
-        search_str_list(hd->drivers, "veth") ||
-        search_str_list(hd->drivers, "iseries_veth")
-      )
-    ) {
-      hd_card = add_hd_entry(hd_data, __LINE__, 0);
-      hd_card->base_class.id = bc_network;
-      hd_card->sub_class.id = 0x00;
-      hd_card->vendor.id = MAKE_ID(TAG_SPECIAL, 0x6001);	// IBM
-      hd_card->device.id = MAKE_ID(TAG_SPECIAL, 0x1000);
-      add_str_list(&hd_card->drivers, "iseries_veth");
-      hd_card->slot = card_cnt++;
-      str_printf(&hd_card->device.name, 0, "Virtual Ethernet card");
-      hd->attached_to = hd_card->idx;
-
-      for(res = hd->res; res; res = res->next) {
-        if(res->any.type == res_hwaddr) break;
-      }
-
-      if(res) {
-        unsigned int slotno;
-
-        res2 = new_mem(sizeof *res2);
-        res2->hwaddr.type = res_hwaddr;
-        res2->hwaddr.addr = new_str(res->hwaddr.addr);
-        add_res_entry(&hd_card->res, res2);
-        if (sscanf(res->hwaddr.addr, "02:01:ff:%x:ff:", &slotno)) {
-          hd_card->slot = slotno;
-          str_printf(&hd_card->device.name, 0, "Virtual Ethernet card %d", hd_card->slot);
-	}
-      }
-
-      add_if_name(hd_card, hd);
-    }
-  }
-
-  if(!card_cnt) {
-    sl0 = read_file("/proc/iSeries/config", 0, 0);
-    for(sl = sl0; sl; sl = sl->next) {
-      if(sscanf(sl->str, "AVAILABLE_VETH=%x", &cardmask) == 1)
-     	 break;
-    }
-    free_str_list(sl0);
-
-    for (i = 0; i < 16; i++) {
-      if ((0x8000 >> i) & cardmask) {
-	hd_card = add_hd_entry(hd_data, __LINE__, 0);
-	hd_card->base_class.id = bc_network;
-	hd_card->sub_class.id = 0x00;
-	hd_card->vendor.id = MAKE_ID(TAG_SPECIAL, 0x6001);	// IBM
-	hd_card->device.id = MAKE_ID(TAG_SPECIAL, 0x1000);
-	hd_card->slot = i;
-	str_printf(&hd_card->device.name, 0, "Virtual Ethernet card %d", i);
-      }
-    }
-  }
-}
 
 
 /*
@@ -731,12 +595,23 @@ void add_kma(hd_data_t *hd_data)
  */
 void add_if_name(hd_t *hd_card, hd_t *hd)
 {
+  str_list_t *sl0;
+
   if(hd->unix_dev_name) {
     if(!search_str_list(hd_card->unix_dev_names, hd->unix_dev_name)) {
-      add_str_list(&hd_card->unix_dev_names, hd->unix_dev_name);
-    }
-    if(!hd_card->unix_dev_name) {
-      hd_card->unix_dev_name = new_str(hd->unix_dev_name);
+      if(hd->sub_class.id == sc_nif_other) {
+        /* add at end */
+        add_str_list(&hd_card->unix_dev_names, hd->unix_dev_name);
+      }
+      else {
+        /* add at top */
+        sl0 = new_mem(sizeof *sl0);
+        sl0->next = hd_card->unix_dev_names;
+        sl0->str = new_str(hd->unix_dev_name);
+        hd_card->unix_dev_names = sl0;
+      }
+      free_mem(hd_card->unix_dev_name);
+      hd_card->unix_dev_name = new_str(hd_card->unix_dev_names->str);
     }
   }
 }
