@@ -51,7 +51,10 @@ static void hd_read_macio(hd_data_t *hd_data);
 static void hd_read_vio(hd_data_t *hd_data);
 static void hd_read_xen(hd_data_t *hd_data);
 static void hd_read_ps3_system_bus(hd_data_t *hd_data);
+static void hd_read_vm(hd_data_t *hd_data);
+static void add_mv643xx_eth(hd_data_t *hd_data, char *entry, char *platform_type);
 static void hd_read_platform(hd_data_t *hd_data);
+static void hd_read_of_platform(hd_data_t *hd_data);
 static void add_xen_network(hd_data_t *hd_data);
 static void add_xen_storage(hd_data_t *hd_data);
 
@@ -77,22 +80,25 @@ void hd_scan_sysfs_pci(hd_data_t *hd_data)
   add_pci_data(hd_data);
 
   PROGRESS(3, 0, "macio");
-
   hd_read_macio(hd_data);
 
   PROGRESS(4, 0, "vio");
-
   hd_read_vio(hd_data);
 
   PROGRESS(5, 0, "xen");
-
   hd_read_xen(hd_data);
-  PROGRESS(6, 0, "ps3");
 
+  PROGRESS(6, 0, "ps3");
   hd_read_ps3_system_bus(hd_data);
   
   PROGRESS(7, 0, "platform");
   hd_read_platform(hd_data);
+
+  PROGRESS(8, 0, "of_platform");
+  hd_read_of_platform(hd_data);
+
+  PROGRESS(9, 0, "vm");
+  hd_read_vm(hd_data);
 }
 
 
@@ -246,21 +252,29 @@ void hd_pci_read_data(hd_data_t *hd_data)
       close(fd);
     }
 
-    str_printf(&s, 0, "%s/edid1", sf_dev);
-    if((fd = open(s, O_RDONLY)) != -1) {
-      pci->edid_len = read(fd, pci->edid, sizeof pci->edid);
+    for(u = 0; u < sizeof pci->edid_len / sizeof *pci->edid_len; u++) {
+      str_printf(&s, 0, "%s/edid%u", sf_dev, u + 1);
+      if((fd = open(s, O_RDONLY)) != -1) {
+        pci->edid_len[u] = read(fd, pci->edid_data[u], sizeof pci->edid_data[u]);
 
-      ADD2LOG("    edid[%u]\n", pci->edid_len);
+        ADD2LOG("    edid%u[%u]\n", u + 1, pci->edid_len[u]);
 
-      if(pci->edid_len > 0) {
-        for(i = 0; i < sizeof pci->edid; i += 0x10) {
-          ADD2LOG("      ");
-          hexdump(&hd_data->log, 1, 0x10, pci->edid + i);
-          ADD2LOG("\n");
+        if(pci->edid_len[u] > 0) {
+          for(i = 0; i < sizeof pci->edid_data[u]; i += 0x10) {
+            ADD2LOG("      ");
+            hexdump(&hd_data->log, 1, 0x10, pci->edid_data[u] + i);
+            ADD2LOG("\n");
+          }
+        }
+        else {
+          pci->edid_len[u] = 0;
         }
-      }
 
-      close(fd);
+        close(fd);
+      }
+      else {
+        pci->edid_len[u] = 0;
+      }
     }
 
     s = free_mem(s);
@@ -661,8 +675,8 @@ void hd_read_macio(hd_data_t *hd_data)
  */
 void hd_read_vio(hd_data_t *hd_data)
 {
-  char *s, *vio_name, *vio_type;
-  int eth_cnt = 0, scsi_cnt = 0;
+  char *s, *vio_devspec, *vio_name, *vio_modalias;
+  int eth_cnt = 0, scsi_cnt = 0, dasd_cnt = 0, cd_cnt = 0;
   hd_t *hd;
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev;
@@ -683,45 +697,61 @@ void hd_read_vio(hd_data_t *hd_data)
       hd_sysfs_id(sf_dev)
     );
 
-    vio_name = vio_type = NULL;
+    vio_devspec = vio_name = vio_modalias = NULL;
 
     if((s = get_sysfs_attr_by_path(sf_dev, "devspec"))) {
-      vio_name = canon_str(s, strlen(s));
-      ADD2LOG("    name = \"%s\"\n", vio_name);
+      vio_devspec = canon_str(s, strlen(s));
+      ADD2LOG("    name = \"%s\"\n", vio_devspec);
     }
 
     if((s = get_sysfs_attr_by_path(sf_dev, "name"))) {
-      vio_type = canon_str(s, strlen(s));
-      ADD2LOG("    type = \"%s\"\n", vio_type);
+      vio_name = canon_str(s, strlen(s));
+      ADD2LOG("    type = \"%s\"\n", vio_name);
+    }
+
+    if((s = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
+      vio_modalias = canon_str(s, strlen(s));
+      ADD2LOG("    modalias = \"%s\"\n", vio_modalias);
     }
 
     if(
-      vio_type && (
-        !strcmp(vio_type, "l-lan") ||
-        !strcmp(vio_type, "v-scsi")
+      vio_name && (
+        !strcmp(vio_name, "l-lan") || /* pseries && iseries */
+        !strcmp(vio_name, "viodasd") || /* iseries */
+        !strcmp(vio_name, "viocd") || /* iseries */
+        !strcmp(vio_name, "v-scsi" /* pseries */)
       )
     ) {
       hd = add_hd_entry(hd_data, __LINE__, 0);
       hd->bus.id = bus_vio;
+      if(vio_modalias) hd->modalias = new_str(vio_modalias);
 
       hd->vendor.id = MAKE_ID(TAG_SPECIAL, 0x6001);
 
-      if(!strcmp(vio_type, "l-lan")) {
+      if(!strcmp(vio_name, "l-lan")) {
         hd->base_class.id = bc_network;
         hd->sub_class.id = 0;	/* ethernet */
         hd->slot = eth_cnt++;
-        hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1002);
         str_printf(&hd->device.name, 0, "Virtual Ethernet card %d", hd->slot);
+      } else if(!strcmp(vio_name, "viodasd")) {
+        hd->base_class.id = bc_storage;
+        hd->sub_class.id = sc_sto_other;
+        hd->slot = dasd_cnt++;
+        str_printf(&hd->device.name, 0, "Virtual DASD %d", hd->slot);
+      } else if(!strcmp(vio_name, "viocd")) {
+        hd->base_class.id = bc_storage;
+        hd->sub_class.id = sc_sto_other;
+        hd->slot = cd_cnt++;
+        str_printf(&hd->device.name, 0, "Virtual CD %d", hd->slot);
       }
       else { /* scsi */
         hd->base_class.id = bc_storage;
         hd->sub_class.id = sc_sto_scsi;
         hd->slot = scsi_cnt++;
-        hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1001);
         str_printf(&hd->device.name, 0, "Virtual SCSI %d", hd->slot);
       }
 
-      hd->rom_id = new_str(vio_name ? vio_name + 1 : 0);	/* skip leading '/' */
+      hd->rom_id = new_str(vio_devspec ? vio_devspec + 1 : 0);	/* skip leading '/' */
 
       hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
       hd->sysfs_bus_id = new_str(sf_bus_e->str);
@@ -735,18 +765,42 @@ void hd_read_vio(hd_data_t *hd_data)
   free_str_list(sf_bus);
 }
 
+
+/*
+ * Marvell Gigabit Ethernet in Pegasos2
+ */
+void add_mv643xx_eth(hd_data_t *hd_data, char *entry, char *platform_type)
+{
+  hd_t *hd;
+  char *sf_dev = NULL;
+
+  str_printf(&sf_dev, 0, "%s/%s", "/sys/devices/platform", entry);
+  ADD2LOG("  platform device: adding %s\n", entry);
+
+  hd = add_hd_entry(hd_data, __LINE__, 0);
+  hd->base_class.id = bc_network;
+  hd->sub_class.id = 0;
+
+  hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+  hd->sysfs_bus_id = new_str(entry);
+  hd->modalias = new_str(platform_type);
+
+  hd->vendor.id = MAKE_ID(TAG_PCI, 0x11ab);
+  hd->device.id = MAKE_ID(TAG_PCI, 0x6460);
+
+  free_mem(sf_dev);
+}
+
+
 /*
  * Get platform data from sysfs.
  */
 void hd_read_platform(hd_data_t *hd_data)
 {
   char *s, *platform_type;
-#if 0
-  int scsi_cnt = 0;
-  hd_t *hd;
-#endif
-  str_list_t *sf_bus, *sf_bus_e;
-  char *sf_dev;
+  str_list_t *sf_bus, *sf_bus_e, *sf_eth_dev = NULL;
+  char *sf_dev, *sf_eth_net;
+  int mv643xx_eth_seen = 0;
 
   sf_bus = reverse_str_list(read_dir("/sys/bus/platform/devices", 'l'));
 
@@ -764,39 +818,25 @@ void hd_read_platform(hd_data_t *hd_data)
       hd_sysfs_id(sf_dev)
     );
 
-    platform_type = NULL;
-
     if((s = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
       platform_type = canon_str(s, strlen(s));
       ADD2LOG("    type = \"%s\"\n", platform_type);
+      sf_eth_net = new_str(hd_read_sysfs_link(sf_dev, "net"));
+      sf_eth_dev = read_dir(sf_eth_net, 'd');
+      ADD2LOG("  platform device: sf_eth_net = %s sf_eth_dev = %p\n", sf_eth_net, sf_eth_dev);
+      if (
+        /* there is 'mv643xx_eth.0', 'mv643xx_eth.1' and 'mv643xx_eth_shared.' */
+        sf_eth_dev && sf_eth_net &&
+        strstr(platform_type, "mv643xx_eth") &&
+        !mv643xx_eth_seen++
+      ) {
+        add_mv643xx_eth(hd_data, sf_bus_e->str, platform_type);
+      }
+      free_mem(platform_type);
+      free_mem(sf_eth_net);
+      free_str_list(sf_eth_dev);
     }
 
-#if 0
-    if(
-      platform_type && !strcmp(platform_type, "ps3_storage")
-    ) {
-      hd = add_hd_entry(hd_data, __LINE__, 0);
-      hd->bus.id = bus_ps3_system_bus;
-
-      hd->vendor.id = MAKE_ID(TAG_PCI, 0x104d); /* Sony */
-
-      hd->base_class.id = bc_storage;
-      hd->sub_class.id = sc_sto_scsi;
-      hd->slot = scsi_cnt++;
-      hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1000); /* PS3_DEV_TYPE_STOR_DISK */
-      str_printf(&hd->device.name, 0, "PS3 storage %d", hd->slot);
-
-      hd->modalias = new_str(platform_type);
-
-      hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
-      hd->sysfs_bus_id = new_str(sf_bus_e->str);
-      s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
-      if(s) add_str_list(&hd->drivers, s);
-    }
-#endif  
-
-    platform_type = free_mem(platform_type);
-
     free_mem(sf_dev);
   }
 
@@ -804,18 +844,115 @@ void hd_read_platform(hd_data_t *hd_data)
 }
 
 
+/*
+ * Get platform data from sysfs.
+ */
+void hd_read_of_platform(hd_data_t *hd_data)
+{
+  char *s, *modalias;
+  str_list_t *sf_bus, *sf_bus_e;
+  hd_t *hd;
+  char *sf_dev;
+
+  sf_bus = reverse_str_list(read_dir("/sys/bus/of_platform/devices", 'l'));
+
+  if(!sf_bus) {
+    ADD2LOG("sysfs: no such bus: of_platform\n");
+    return;
+  }
+
+  for(sf_bus_e = sf_bus; sf_bus_e; sf_bus_e = sf_bus_e->next) {
+    sf_dev = new_str(hd_read_sysfs_link("/sys/bus/of_platform/devices", sf_bus_e->str));
+    ADD2LOG(
+      "  of_platform device: name = %s\n    path = %s\n",
+      sf_bus_e->str, hd_sysfs_id(sf_dev)
+    );
+    if((modalias = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
+      int len = strlen(modalias);
+      if (len > 0 && modalias[len - 1] == '\n')
+	      modalias[len - 1] = '\0';
+      ADD2LOG("    modalias = \"%s\"\n", modalias);
+      if (0) ;
+      else if (strstr(modalias, "Cmpc5200-fec")) {
+		/* EFIKA52K network */
+          hd = add_hd_entry(hd_data, __LINE__, 0);
+
+          hd->vendor.id = MAKE_ID(TAG_PCI, 0x1957); /* Freescale */
+          hd->base_class.id = bc_network;
+          hd->sub_class.id = 0;	/* ethernet */
+          str_printf(&hd->device.name, 0, "mpc5200 Ethernet %d", hd->slot);
+
+          hd->modalias = new_str(modalias);
+
+          hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+          hd->sysfs_bus_id = new_str(sf_bus_e->str);
+          s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+          if(s) add_str_list(&hd->drivers, s);
+      } else if (strstr(modalias, "Cmpc5200-ata")) {
+		/* EFIKA52K SATA */
+          hd = add_hd_entry(hd_data, __LINE__, 0);
+
+          hd->vendor.id = MAKE_ID(TAG_PCI, 0x1957); /* Freescale */
+          hd->base_class.id = bc_storage;
+          hd->sub_class.id = sc_sto_ide;	/* 2.5" disk */
+          str_printf(&hd->device.name, 0, "mpc5200 SATA %d", hd->slot);
+
+          hd->modalias = new_str(modalias);
+
+          hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+          hd->sysfs_bus_id = new_str(sf_bus_e->str);
+          s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+          if(s) add_str_list(&hd->drivers, s);
+      } else if (strstr(modalias, "Cmpc5200-psc-ac97")) {
+		/* EFIKA52K sound */
+          hd = add_hd_entry(hd_data, __LINE__, 0);
+
+          hd->vendor.id = MAKE_ID(TAG_PCI, 0x1957); /* Freescale */
+          hd->base_class.id = bc_multimedia;
+          hd->sub_class.id = sc_multi_audio;
+          str_printf(&hd->device.name, 0, "mpc5200 AC97 %d", hd->slot);
+
+          hd->modalias = new_str(modalias);
+
+          hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+          hd->sysfs_bus_id = new_str(sf_bus_e->str);
+          s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+          if(s) add_str_list(&hd->drivers, s);
+      } else if (strstr(modalias, "Cmpc5200-usb")) {
+		/* EFIKA52K USB */
+          hd = add_hd_entry(hd_data, __LINE__, 0);
+
+          hd->vendor.id = MAKE_ID(TAG_PCI, 0x1957); /* Freescale */
+          hd->base_class.id = bc_serial;
+          hd->sub_class.id = sc_ser_usb;
+          hd->prog_if.id = pif_usb_ohci;
+
+          str_printf(&hd->device.name, 0, "mpc5200 USB %d", hd->slot);
+
+          hd->modalias = new_str(modalias);
+
+          hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+          hd->sysfs_bus_id = new_str(sf_bus_e->str);
+          s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+          if(s) add_str_list(&hd->drivers, s);
+      }
+    }
+    free_mem(sf_dev);
+  }
+  free_str_list(sf_bus);
+}
+
 
 /*
  * Get ps3 data from sysfs.
  */
-static void hd_read_ps3_system_bus(hd_data_t *hd_data)
+void hd_read_ps3_system_bus(hd_data_t *hd_data)
 {
   char *s, *ps3_name;
-  int eth_cnt = 0; 
-  int scsi_cnt = 0;
+  int scsi_cnt = 0, eth_cnt = 0, wlan_cnt = 0;
   hd_t *hd;
-  str_list_t *sf_bus, *sf_bus_e;
-  char *sf_dev;
+  str_list_t *sf_bus, *sf_bus_e, *sf_eth_dev, *sf_eth_dev_e;
+  char *sf_dev, *sf_eth_net, *sf_eth_wireless;
 
   sf_bus = reverse_str_list(read_dir("/sys/bus/ps3_system_bus/devices", 'l'));
 
@@ -840,29 +977,41 @@ static void hd_read_ps3_system_bus(hd_data_t *hd_data)
       ADD2LOG("    modalias = \"%s\"\n", ps3_name);
     }
 
-    if(
-      ps3_name && (
-        !strcmp(ps3_name, "ps3:3") ||
-        !strcmp(ps3_name, "gelic_net")
-      )
-    ) {
-      hd = add_hd_entry(hd_data, __LINE__, 0);
-      hd->bus.id = bus_ps3_system_bus;
+    /* network devices */
+    if(ps3_name && !strcmp(ps3_name, "ps3:3")) {
+      /* read list of available devices */
+      sf_eth_net = new_str(hd_read_sysfs_link(sf_dev, "net"));
+      sf_eth_dev = read_dir(sf_eth_net, 'd');
 
-      hd->vendor.id = MAKE_ID(TAG_PCI, 0x104d); /* Sony */
+      /* add entries for available devices */
+      for(sf_eth_dev_e = sf_eth_dev; sf_eth_dev_e; sf_eth_dev_e = sf_eth_dev_e->next) {
+        hd = add_hd_entry(hd_data, __LINE__, 0);
+        hd->bus.id = bus_ps3_system_bus;
+        hd->sysfs_bus_id = new_str(sf_bus_e->str);
+        hd->slot = eth_cnt + wlan_cnt;
+        hd->vendor.id = MAKE_ID(TAG_PCI, 0x104d);		/* Sony */
+        hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1003);		/* PS3_DEV_TYPE_SB_GELIC */
+        hd->base_class.id = bc_network;
+        hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+        hd->modalias = new_str(ps3_name);
+        hd->unix_dev_name = new_str(sf_eth_dev_e->str);		/* this is needed to correctly link to interfaces later */
 
-      hd->base_class.id = bc_network;
-      hd->sub_class.id = 0;	/* ethernet */
-      hd->slot = eth_cnt++;
-      hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1003); /* PS3_DEV_TYPE_SB_GELIC */
-      str_printf(&hd->device.name, 0, "PS3 Ethernet card %d", hd->slot);
+        /* ethernet and wireless differ only by directory "wireless" so check for it */
+        sf_eth_wireless = hd_read_sysfs_link(hd_read_sysfs_link(sf_eth_net, sf_eth_dev_e->str), "wireless");
+        if(sf_eth_wireless) {
+          hd->sub_class.id = 0x82;	/* wireless */
+          str_printf(&hd->device.name, 0, "PS3 Wireless card %d", wlan_cnt++);
+        }
+        else {
+          hd->sub_class.id = 0;		/* ethernet */
+          str_printf(&hd->device.name, 0, "PS3 Ethernet card %d", eth_cnt++);
+        }
+        s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+        if(s) add_str_list(&hd->drivers, s);
+      }
 
-      hd->modalias = new_str(ps3_name);
-
-      hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
-      hd->sysfs_bus_id = new_str(sf_bus_e->str);
-      s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
-      if(s) add_str_list(&hd->drivers, s);
+      sf_eth_net = free_mem(sf_eth_net);
+      sf_eth_dev = free_str_list(sf_eth_dev);
     }
 
     if ( ps3_name && !strcmp(ps3_name, "ps3:7")) {
@@ -904,6 +1053,25 @@ static void hd_read_ps3_system_bus(hd_data_t *hd_data)
       s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
       if(s) add_str_list(&hd->drivers, s);
 
+    }
+    if ( ps3_name && !strcmp(ps3_name, "ps3:9")) {
+      hd = add_hd_entry(hd_data, __LINE__, 0);
+      hd->bus.id = bus_ps3_system_bus;
+
+      hd->vendor.id = MAKE_ID(TAG_PCI, 0x104d); /* Sony */
+
+      hd->base_class.id = bc_multimedia;
+      hd->sub_class.id = sc_multi_audio;
+      hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1004);
+      str_printf(&hd->device.name, 0, "PS3 Soundcard");
+
+      hd->modalias = new_str(ps3_name);
+
+      hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+      hd->sysfs_bus_id = new_str(sf_bus_e->str);
+      s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+      if(s) add_str_list(&hd->drivers, s);
+
     }
     ps3_name = free_mem(ps3_name);
 
@@ -913,6 +1081,7 @@ static void hd_read_ps3_system_bus(hd_data_t *hd_data)
   free_str_list(sf_bus);
 }
 
+
 /*
  * Get xen (network & storage) data from sysfs.
  */
@@ -1057,5 +1226,68 @@ void add_xen_storage(hd_data_t *hd_data)
 }
 
 
+/*
+ * Get microsoft vm (network) data from sysfs.
+ */
+void hd_read_vm(hd_data_t *hd_data)
+{
+  int eth_cnt = 0;
+  hd_t *hd;
+  str_list_t *sf_bus, *sf_bus_e;
+  char *sf_dev, *drv, *drv_name;
+
+  sf_bus = reverse_str_list(read_dir("/sys/bus/vmbus/devices", 'l'));
+
+  if(!sf_bus) {
+    ADD2LOG("sysfs: no such bus: vm\n");
+    return;
+  }
+
+  for(sf_bus_e = sf_bus; sf_bus_e; sf_bus_e = sf_bus_e->next) {
+    sf_dev = new_str(hd_read_sysfs_link("/sys/bus/vmbus/devices", sf_bus_e->str));
+
+    ADD2LOG(
+      "  vm device: name = %s\n    path = %s\n",
+      sf_bus_e->str,
+      hd_sysfs_id(sf_dev)
+    );
+
+    drv_name = NULL;
+    drv = new_str(hd_read_sysfs_link(sf_dev, "driver"));
+    if(drv) {
+      drv_name = strrchr(drv, '/');
+      if(drv_name) drv_name++;
+    }
+
+    ADD2LOG("    driver = \"%s\"\n", drv_name);
+
+    if(
+      drv_name &&
+      !strcmp(drv_name, "netvsc")
+    ) {
+      hd = add_hd_entry(hd_data, __LINE__, 0);
+      hd->bus.id = bus_none;
+
+      hd->vendor.id = MAKE_ID(TAG_SPECIAL, 0x6013);	/* virtual */
+
+      hd->base_class.id = bc_network;
+      hd->sub_class.id = 0;	/* ethernet */
+      hd->slot = eth_cnt++;
+      hd->device.id = MAKE_ID(TAG_SPECIAL, 1);
+      str_printf(&hd->device.name, 0, "Virtual Ethernet Card %d", hd->slot);
+
+      hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+      hd->sysfs_bus_id = new_str(sf_bus_e->str);
+      if(drv_name) add_str_list(&hd->drivers, drv_name);
+    }
+
+    free_mem(sf_dev);
+    free_mem(drv);
+  }
+
+  free_str_list(sf_bus);
+}
+
+
 /** @} */
 
