@@ -13,14 +13,14 @@
 #include "hd_int.h"
 #include "pcmcia.h"
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- *
- * pcmcia via sysfs
- *
+/**
+ * @defgroup PCMCIAint PCMCIA (via SysFS)
+ * @ingroup libhdBUSint
+ * @brief PCMCIA bus scan functions
  *
  * expects pci devs to be probed already to assign bridge
  *
- * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ * @{
  */
 
 static void pcmcia_read_data(hd_data_t *hd_data);
@@ -60,34 +60,31 @@ void pcmcia_read_data(hd_data_t *hd_data)
   char *s, *t;
   char *prod1, *prod2, *prod3, *prod4;
   str_list_t *sl;
+  str_list_t *sf_bus, *sf_bus_e;
+  char *sf_dev;
 
-  struct sysfs_bus *sf_bus;
-  struct dlist *sf_dev_list;
-  struct sysfs_device *sf_dev;
-
-  sf_bus = sysfs_open_bus("pcmcia");
+  sf_bus = reverse_str_list(read_dir("/sys/bus/pcmcia/devices", 'l'));
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: pcmcia\n");
     return;
   }
 
-  sf_dev_list = sysfs_get_bus_devices(sf_bus);
-  if(sf_dev_list) dlist_for_each_data(sf_dev_list, sf_dev, struct sysfs_device) {
+  for(sf_bus_e = sf_bus; sf_bus_e; sf_bus_e = sf_bus_e->next) {
+    sf_dev = new_str(hd_read_sysfs_link("/sys/bus/pcmcia/devices", sf_bus_e->str));
+
     ADD2LOG(
-      "  pcmcia device: name = %s, bus_id = %s, bus = %s\n    path = %s\n",
-      sf_dev->name,  
-      sf_dev->bus_id,
-      sf_dev->bus,
-      hd_sysfs_id(sf_dev->path)
+      "  pcmcia device: name = %s\n    path = %s\n",
+      sf_bus_e->str,
+      hd_sysfs_id(sf_dev)
     );
 
-    if(sscanf(sf_dev->bus_id, "%x.%x", &u0, &u1) != 2) continue;
+    if(sscanf(sf_bus_e->str, "%x.%x", &u0, &u1) != 2) continue;
 
     hd = add_hd_entry(hd_data, __LINE__, 0);
 
-    hd->sysfs_id = new_str(hd_sysfs_id(sf_dev->path));
-    hd->sysfs_bus_id = new_str(sf_dev->bus_id);
+    hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+    hd->sysfs_bus_id = new_str(sf_bus_e->str);
     hd->bus.id = bus_pcmcia;
 
     hd->slot = u0;
@@ -98,17 +95,17 @@ void pcmcia_read_data(hd_data_t *hd_data)
     s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
     if(s) add_str_list(&hd->drivers, s);
 
-    if((s = hd_attr_str(sysfs_get_device_attr(sf_dev, "modalias")))) {
+    if((s = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
       hd->modalias = canon_str(s, strlen(s));
       ADD2LOG("    modalias = \"%s\"\n", s);
     }
 
-    if(hd_attr_uint(sysfs_get_device_attr(sf_dev, "manf_id"), &ul0, 0)) {
+    if(hd_attr_uint(get_sysfs_attr_by_path(sf_dev, "manf_id"), &ul0, 0)) {
       ADD2LOG("    manf_id = 0x%04x\n", (unsigned) ul0);
       hd->vendor.id = MAKE_ID(TAG_PCMCIA, ul0);
     }
 
-    if(hd_attr_uint(sysfs_get_device_attr(sf_dev, "card_id"), &ul0, 0)) {
+    if(hd_attr_uint(get_sysfs_attr_by_path(sf_dev, "card_id"), &ul0, 0)) {
       ADD2LOG("    card_id = 0x%04x\n", (unsigned) ul0);
       hd->device.id = MAKE_ID(TAG_PCMCIA, ul0);
     }
@@ -119,31 +116,31 @@ void pcmcia_read_data(hd_data_t *hd_data)
      * "SCSI"
      */
     func_id = 0;
-    if(hd_attr_uint(sysfs_get_device_attr(sf_dev, "func_id"), &ul0, 0)) {
+    if(hd_attr_uint(get_sysfs_attr_by_path(sf_dev, "func_id"), &ul0, 0)) {
       func_id = ul0;
       ADD2LOG("    func_id = 0x%04x\n", func_id);
     }
 
     prod1 = NULL;
-    if((s = hd_attr_str(sysfs_get_device_attr(sf_dev, "prod_id1")))) {
+    if((s = get_sysfs_attr_by_path(sf_dev, "prod_id1"))) {
       prod1 = canon_str(s, strlen(s));
       ADD2LOG("    prod_id1 = \"%s\"\n", prod1);
     }
 
     prod2 = NULL;
-    if((s = hd_attr_str(sysfs_get_device_attr(sf_dev, "prod_id2")))) {
+    if((s = get_sysfs_attr_by_path(sf_dev, "prod_id2"))) {
       prod2 = canon_str(s, strlen(s));
       ADD2LOG("    prod_id2 = \"%s\"\n", prod2);
     }
 
     prod3 = NULL;
-    if((s = hd_attr_str(sysfs_get_device_attr(sf_dev, "prod_id3")))) {
+    if((s = get_sysfs_attr_by_path(sf_dev, "prod_id3"))) {
       prod3 = canon_str(s, strlen(s));
       ADD2LOG("    prod_id3 = \"%s\"\n", prod3);
     }
 
     prod4 = NULL;
-    if((s = hd_attr_str(sysfs_get_device_attr(sf_dev, "prod_id4")))) {
+    if((s = get_sysfs_attr_by_path(sf_dev, "prod_id4"))) {
       prod4 = canon_str(s, strlen(s));
       ADD2LOG("    prod_id4 = \"%s\"\n", prod4);
     }
@@ -188,7 +185,6 @@ void pcmcia_read_data(hd_data_t *hd_data)
     prod3 = free_mem(prod3);
     prod4 = free_mem(prod4);
 
-
     s = new_str(hd->sysfs_id);
 
     if((t = strrchr(s, '/'))) {
@@ -199,9 +195,10 @@ void pcmcia_read_data(hd_data_t *hd_data)
     }
     free_mem(s);
 
+    free_mem(sf_dev);
   }
 
-  sysfs_close_bus(sf_bus);
+  free_str_list(sf_bus);
 }
 
 
@@ -211,41 +208,42 @@ void pcmcia_ctrl_read_data(hd_data_t *hd_data)
   char *s;
   unsigned u;
   unsigned sockets[16 /* just large enough */] = { };
+  str_list_t *sf_class, *sf_class_e;
+  char *sf_cdev = NULL, *sf_dev;
 
-  struct sysfs_class *sf_class;
-  struct sysfs_class_device *sf_cdev;
-  struct sysfs_device *sf_dev;
-  struct dlist *sf_cdev_list;
-
-  sf_class = sysfs_open_class("pcmcia_socket");
+  sf_class = reverse_str_list(read_dir("/sys/class/pcmcia_socket", 'd'));
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: pcmcia_socket\n");
   }
   else {
-    sf_cdev_list = sysfs_get_class_devices(sf_class);
+    for(sf_class_e = sf_class; sf_class_e; sf_class_e = sf_class_e->next) {
+      str_printf(&sf_cdev, 0, "/sys/class/pcmcia_socket/%s", sf_class_e->str);
+      sf_dev = new_str(hd_read_sysfs_link(sf_cdev, "device"));
 
-    if(sf_cdev_list) dlist_for_each_data(sf_cdev_list, sf_cdev, struct sysfs_class_device) {
       if(
-        sscanf(sf_cdev->name, "pcmcia_socket%u", &u) == 1 &&
-        (sf_dev = sysfs_get_classdev_device(sf_cdev))
+        sf_dev &&
+        sscanf(sf_class_e->str, "pcmcia_socket%u", &u) == 1
       ) {
-        s = hd_sysfs_id(sf_dev->path);
+        s = hd_sysfs_id(sf_dev);
         hd = hd_find_sysfs_id(hd_data, s);
         if(hd && u < sizeof sockets / sizeof *sockets) sockets[u] = hd->idx;
 
         ADD2LOG("  pcmcia socket %u: %s\n", u, s);
       }
+
+      free_mem(sf_dev);
     }
 
-    sysfs_close_class(sf_class);
+    sf_cdev = free_mem(sf_cdev);
   }
 
+  sf_class = free_str_list(sf_class);
 
   /* find card bus devices & assign them socket numbers */
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
-    if(hd->bus.id != bus_pci) continue;
+    if(hd->bus.id != bus_pcmcia) continue;
     if((bridge_hd = hd_get_device_by_idx(hd_data, hd->attached_to))) {
       if(
         bridge_hd->base_class.id == bc_bridge &&
@@ -269,4 +267,5 @@ void pcmcia_ctrl_read_data(hd_data_t *hd_data)
   }
 }
 
+/** @} */
 
