@@ -21,7 +21,7 @@
  * @{
  */
 
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
 static void sigsegv_handler(int signum);
 static void chk_vmware(hd_data_t *hd_data, sys_info_t *st);
 #endif
@@ -113,7 +113,7 @@ void hd_scan_sys(hd_data_t *hd_data)
   chk_vaio(hd_data, st);
 #endif
 
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
   chk_vmware(hd_data, st);
 #endif
 
@@ -126,7 +126,7 @@ void hd_scan_sys(hd_data_t *hd_data)
   }
 }
 
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
 void sigsegv_handler(int signum) { _exit(77); }
 
 void chk_vmware(hd_data_t *hd_data, sys_info_t *st)
@@ -142,6 +142,7 @@ void chk_vmware(hd_data_t *hd_data, sys_info_t *st)
     if(child == 0) {
       signal(SIGSEGV, sigsegv_handler);
 
+#ifdef __i386__
       asm(
         "push %ebx\n"
         "\tpush %edx\n"
@@ -156,6 +157,22 @@ void chk_vmware(hd_data_t *hd_data, sys_info_t *st)
         "\tpop %edx\n"
         "\tpop %ebx\n"
       );
+#else
+      asm(
+        "push %rbx\n"
+        "\tpush %rdx\n"
+        "\tpush %rax\n"
+        "\tpush %rcx\n"
+        "\tmov $0x564d5868,%eax\n"
+        "\tmov $0xa,%ecx\n"
+        "\tmov $0x5658,%edx\n"
+        "\tin (%dx),%eax\n"
+        "\tpop %rcx\n"
+        "\tpop %rax\n"
+        "\tpop %rdx\n"
+        "\tpop %rbx\n"
+      );
+#endif
 
       _exit(66);
     }
@@ -174,10 +191,10 @@ void chk_vmware(hd_data_t *hd_data, sys_info_t *st)
     st->model = new_str("VMWare");
   }
 
-  hd_data->in_vmware = is_vmware;
+  hd_data->flags.vmware = is_vmware;
 }
 
-#endif	/* __i386__ */
+#endif	/* __i386__ || __x86_64__ */
 
 
 #if defined(__i386__) || defined(__x86_64__)
