@@ -386,13 +386,18 @@ void set_class_entries(hd_data_t *hd_data, hd_t *hd, usb_t *usb)
 
     case 3:
       if(sub == 1 && prot == 1) {
-        hd->base_class.id = bc_keyboard;
-        hd->sub_class.id = sc_keyboard_kbd;
+        if(!(
+          (usb->vendor == 0x05ac && usb->device == 0x1000)	/* MacBook, bnc #374101 */
+        )) {
+          hd->base_class.id = bc_keyboard;
+          hd->sub_class.id = sc_keyboard_kbd;
+        }
         break;
       }
       if(sub == 1 && prot == 2) {
         if(!(
           (usb->vendor == 0x056a && usb->device == 0x0022)	/* Wacom Tablet */
+          || (usb->vendor == 0x147a && usb->device == 0xe001)	/* eDio USB Multi Remote Controlle */
 //          || (usb->vendor == 0x08ca && usb->device == 0x0020)	/* AIPTEK APT-6000U tablet */
         )) {
           hd->base_class.id = bc_mouse;
@@ -518,6 +523,13 @@ void add_input_dev(hd_data_t *hd_data, char *name)
   sf_dev = new_str(hd_read_sysfs_link(name, "device"));
 
   if(sf_dev) {
+    /* new kernel (2.6.24): one more level */
+    s = new_str(hd_read_sysfs_link(sf_dev, "device"));
+    if(s) {
+      free_mem(sf_dev);
+      sf_dev = s;
+    }
+
     bus_id = sf_dev ? strrchr(sf_dev, '/') : NULL;
     if(bus_id) bus_id++;
 
@@ -640,7 +652,8 @@ void get_printer_devs(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev;
   char *sf_drv_name, *sf_drv, *bus_id, *bus_name;
 
-  sf_class = reverse_str_list(read_dir("/sys/class/usb", 'd'));
+  sf_class = reverse_str_list(read_dir("/sys/class/usb", 'D'));
+  if(!sf_class) sf_class = reverse_str_list(read_dir("/sys/class/usb_endpoint", 'l'));
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: usb\n");
@@ -808,7 +821,7 @@ void get_serial_devs(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev;
   char *sf_drv_name, *sf_drv, *bus_id, *bus_name;
 
-  sf_class = reverse_str_list(read_dir("/sys/class/tty", 'd'));
+  sf_class = reverse_str_list(read_dir("/sys/class/tty", 'D'));
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: tty\n");
